"""Mixin classes for unittests"""
from chromatictools import math
import functools
import contextlib
import numpy as np
import io


class AssertPrintsMixin:
  """Mixin class for print assertion"""
  @contextlib.contextmanager
  def assert_prints(self, target: str):
    """Assert that contextual code prints the target string.
    Use as context manager

    Args:
      target (str): Print expectation"""
    with io.StringIO() as buf, contextlib.redirect_stdout(buf):
      yield
      printed = buf.getvalue()
    self.assertEqual(printed, target)


class AssertDoesntRaiseMixin:
  """Mixin class for "doesn't-raise" assertion"""
  @contextlib.contextmanager
  def assert_doesnt_raise(self):
    """Assert that contextual code doesn't raise any exception
    Use as context manager"""
    try:
      yield
    except Exception as e:
      raise AssertionError from e


class SignificantPlacesAssertMixin:
  """Mixin class for assertions based on singificant places"""
  def assert_almost_equal_significant(
    self,
    first,
    second,
    places=0,
    msg=None,
    delta=None
  ):
    """Fail if the two objects are unequal as determined by their
    difference rounded to the given number of significant decimal places
    (default 0)"""
    return self.assertAlmostEqual(
      *math.mantissa([first, second]),
      places=places,
      msg=msg,
      delta=delta
    )


class RMSEAssertMixin:
  """Mixin class for assertions based on the RMSE"""
  def _assert_rmse(
    self,
    x: np.ndarray,
    y: np.ndarray,
    almost: bool,
    rmse: float = 0,
    *args,
    **kwargs):
    """Assert that the RMSE is equal to a given value

    Arguments:
      x (array): First array
      y (array): Second array
      almost (bool): If True, then assert that the RMSE is almost zero
      rmse (float): RMSE value. Defaults to 0
      *args: Positional arguments for :func:`unittest.TestCase.assertEqual`
        or :func:`unittest.TestCase.assertAlmostEqual`
      **kwargs: Keyword arguments for :func:`unittest.TestCase.assertEqual`
        or :func:`unittest.TestCase.assertAlmostEqual`"""
    return (self.assertAlmostEqual if almost else self.assertEqual)(
      math.rmse(x, y), rmse,
      *args, **kwargs,
    )

  assert_equal_rmse = functools.partialmethod(_assert_rmse, almost=False)
  assert_almost_equal_rmse = functools.partialmethod(_assert_rmse, almost=True)



"""Math utilities"""
import numpy as np


def mantissa(x: np.array) -> np.ndarray:
  """Remove order of magnitude from array

  Args:
  x (array): Inputs

  Returns:
    array: Rescaled array"""
  y = np.log10(np.abs(x))
  return np.sign(x) * np.power(10., (y - np.floor(np.max(y))))


def rmse(x: np.ndarray, y: np.ndarray) -> float:
  """Compute the root-mean-squared error

  Arguments:
    x (array): First array
    y (array): Second array

  Returns:
    (float): The RMSE between x and y"""
  return np.sqrt(np.mean(np.abs(x - y)**2))



"""Purge GitHub page image caches"""
from chromatictools import cli
import argparse
import requests
import subprocess
import sys
import re


@cli.main(__name__, *sys.argv[1:])
def main(*argv):
  """CLI launcher function

  Args:
    argv: Command-line arguments

  Returns:
    int: Exit code"""
  parser = argparse.ArgumentParser(description=globals()["__doc__"])
  parser.add_argument("url", help="GitHub URL")
  parser.add_argument(
    "-p", "--pattern",
    metavar="pattern",
    dest="pattern",
    default="<img src=\"https://camo\\.githubusercontent\\.com.*\"",
    help="Pattern for image search"
  )
  parser.add_argument(
    "-l", "--left",
    metavar="delimiter",
    dest="left",
    default="\"",
    help="Left delimiter to clean pattern"
  )
  parser.add_argument(
    "-r", "--right",
    metavar="delimiter",
    dest="right",
    default="\"",
    help="Right delimiter to clean pattern"
  )
  args = parser.parse_args(argv)

  r = requests.get(args.url)
  urls = map(
    lambda s: s.split(args.left, 1)[1].split(args.right, 1)[0],
    re.findall(args.pattern, r.text)
  )
  cmd = ("curl", "-X", "PURGE")
  for url in urls:
    print(*cmd, url)
    with subprocess.Popen(
      [*cmd, url],
      stdout=subprocess.PIPE,
      stderr=subprocess.PIPE,
    ) as p:
      s = p.stdout.read().decode()
    print(s, "\n")
